@@ -1,19 +1,23 @@
 from datetime import datetime, timedelta
+import sys
+from tqdm import tqdm as tqdm_original
+from collections import OrderedDict
+from numbers import Number
 
 
 class APBar:
     def __init__(
-        self, iterator, period: int = 10, sensitivity: int = 1, bar_width: int = 10
+        self, iterable, period: int = 60, sensitivity: int = 1, bar_width: int = 10
     ) -> None:
         """
         Class constructor
-         - iterator
-         - period: 10, time between updates in seconds
+         - iterable
+         - period: 60, time between updates in seconds
          - sensitivity: 1, minimum difference vs previous progress to show
          - bar_width: 10, width of the progress bar pseudographics
         """
-        self.iter = iter(iterator)
-        self.len = len(iterator)
+        self.iter = iter(iterable)
+        self.len = len(iterable)
         self.maxdigits = len(str(self.len)) * 2 + 3
         self.step = 0
         self.last = -1
@@ -53,19 +57,59 @@ class APBar:
         output += "]"
         return output
 
-    def set_description_str(self, string):
+    def set_description(self, desc=None, refresh=True):
+        """
+        Set/modify description of the progress bar.
+
+        Parameters
+        ----------
+        desc  : str, optional
+        refresh  : bool, ignored
+        """
+        self.prefix = desc + ": " if desc else ""
+
+    def set_description_str(self, desc):
         """
         Method to set text preceding pseudographics bar
-         - string: string to show at next(!) iteration
+         - desc: string to show at next(!) iteration
         """
-        self.prefix = string + ":"
+        self.prefix = desc
 
-    def set_postfix_str(self, string):
+    def set_postfix(self, ordered_dict=None, refresh=True, **kwargs):
+        """
+        Set/modify postfix (additional stats)
+        with automatic formatting based on datatype.
+
+        Parameters
+        ----------
+        ordered_dict  : dict or OrderedDict, optional
+        refresh  : bool, ignored
+        kwargs  : dict, optional
+        """
+        # Sort in alphabetical order to be more deterministic
+        postfix = OrderedDict([] if ordered_dict is None else ordered_dict)
+        for key in sorted(kwargs.keys()):
+            postfix[key] = kwargs[key]
+        # Preprocess stats according to datatype
+        for key in postfix.keys():
+            # Number: limit the length of the string
+            if isinstance(postfix[key], Number):
+                postfix[key] = self.format_num(postfix[key])
+            # Else for any other type, try to get the string conversion
+            elif not isinstance(postfix[key], str):
+                postfix[key] = str(postfix[key])
+            # Else if it's a string, don't need to preprocess anything
+        # Stitch together to get the final postfix
+        self.postfix = ", ".join(
+            key + "=" + postfix[key].strip() for key in postfix.keys()
+        )
+
+    def set_postfix_str(self, s):
         """
         Method to set text at the end of the string
-         - string: string to show at next(!) iteration
+         - s: string to show at next(!) iteration
         """
-        self.postfix = "#" + string
+        self.postfix = "#" + str(s)
 
     def pp(
         self,
@@ -124,11 +168,39 @@ class APBar:
         return next(self.iter)
 
 
-def tqdm(iterator):
+def tqdm(*args, period: int = 60, sensitivity: int = 1, bar_width: int = 10, **kwargs):
     """
-    Function to create instance of a APBar class
+    tqdm selector function. Returns regular tqdm if no running wandb or comet ml instances detected, otherwise returns custom status bar with less frequent updates
+    Arguments
+        - iterable
+        - period: 60, time between updates in seconds
+        - sensitivity: 1, minimum difference vs previous progress to show
+        - bar_width: 10, width of the progress bar pseudographics
     """
-    return APBar(iterator=iterator)
+    try:
+        if (
+            "wandb" in sys.modules
+            and hasattr(sys.modules["wandb"], "run")
+            and sys.modules["wandb"].run is not None
+        ) or (
+            "comet_ml" in sys.modules
+            and sys.modules["comet_ml"].get_global_experiment() is not None
+        ):
+            return APBar(
+                iterable=args[0],
+                period=period,
+                sensitivity=sensitivity,
+                bar_width=bar_width,
+            )
+    except:
+        pass
+    if "period" in kwargs:
+        kwargs.pop("period")
+    if "sensitivity" in kwargs:
+        kwargs.pop("sensitivity")
+    if "bar_width" in kwargs:
+        kwargs.pop("bar_width")
+    return tqdm_original(*args, **kwargs)
 
 
 if __name__ == "__main__":
